# -*- coding: UTF-8 -*-
# Copyright 2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

from django.db import models
from django.conf import settings
from django.utils.html import format_html, mark_safe
from etgen import html as xghtml
from lino.utils.html import E, join_elems, tostring, SAFE_EMPTY
from lino.core import constants
from lino.modlib.users.mixins import UserAuthored, My
from lino.modlib.system.choicelists import DisplayColors
from lino.mixins import Referrable, Sequenced, Created
from lino.api import dd, rt, _, gettext

from .roles import PrimaStaff, PrimaTeacher, PrimaPupil


class Grades(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Grade'
    detail_layout = """
    id ref
    designation
    projects.ProjectTemplatesByGrade GroupsByGrade
    """

class SubjectDetail(dd.DetailLayout):
    main = "general CoursesBySubject"
    general = dd.Panel("""
    designation id advanced
    icon_text image_file rating_type 
    SkillsBySubject
    """, _("General"))


class Subjects(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Subject'
    detail_layout = "prima.SubjectDetail"


class Skills(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Skill'
    detail_layout = """
    designation id
    subject with_exams #with_smilies
    projects.ProjectTemplatesBySkill ratings.ChallengesBySkill cert.RatingsBySkill
    """

class SkillsBySubject(Skills):
    master_key = "subject"
    column_names = "designation with_exams *"
    default_display_modes = { None: constants.DISPLAY_MODE_GRID}

class GroupDetail(dd.DetailLayout):
    main = "general ratings.ExamsByGroup ratings.SkillsByGroup #scores more"

    general = dd.Panel("""
    designation
    CoursesByGroup CastsByGroup
    projects.PupilsAndProjectsByGroup
    """, _("General"))

    # exams = dd.Panel("""
    # year grade id
    # EnrolmentsByGroup
    # """, _("More"))

    # scores = dd.Panel("""
    # ratings.SkillsByGroup
    # ratings.ChallengeScoresByGroup ratings.PupilScoresByGroup
    # """, _("Scores"))

    more = dd.Panel("""
    year grade id
    EnrolmentsByGroup
    # ExamsByGroup
    """, _("More"))


class Groups(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Group'
    detail_layout = "prima.GroupDetail"
    column_names = "designation grade year id *"
    # default_display_modes = {None: constants.DISPLAY_MODE_SUMMARY}
    parameters = dict(
        teacher=dd.ForeignKey('users.User', verbose_name=_("Teacher"), blank=True, null=True),
        pupil=dd.ForeignKey('users.User', verbose_name=_("Pupil"), blank=True, null=True)
    )

class GroupsByGrade(Groups):
    master_key = "grade"

# class GroupsByTeacher(Groups):
#     master_key = "cast__user"

class MyGroups(Groups):
    required_roles = dd.login_required((PrimaTeacher, PrimaPupil))
    label = _("My groups")
    default_display_modes = {None: constants.DISPLAY_MODE_LIST}

    @classmethod
    def param_defaults(self, ar, **kw):
        kw = super().param_defaults(ar, **kw)
        me = ar.get_user()
        if me.user_type.has_required_roles([dd.SiteAdmin]):
            return kw
        if me.user_type.has_required_roles([PrimaTeacher]):
            kw["teacher"] = me
        elif me.user_type.has_required_roles([PrimaPupil]):
            kw["pupil"] = me
        return kw

    @classmethod
    def get_request_queryset(self, ar, **filter):
        qs = super().get_request_queryset(ar, **filter)
        if (pv := ar.param_values) is None:
            return qs
        # raise Exception(f"20241210 {pv}")
        if pv.teacher is not None:
            qs = qs.filter(cast__user=pv.teacher).distinct()
        if pv.pupil is not None:
            qs = qs.filter(enrolment__pupil=pv.pupil).distinct()
        # print(f"20241016 {qs.explain()}")
        # print(f"20241016 {qs.query}")
        return qs

    @classmethod
    def row_as_paragraph(cls, ar, row):
        s = ar.obj2htmls(row)
        sar = CoursesByGroup.request(row, parent=ar)
        items = [sar.obj2htmls(o) for o in sar]
        if len(items):
            s += ": " + ", ".join(items)
        # else:
        #     s += ": " + _("No courses in {group}").format(group=row)
        return mark_safe(s)


class Roles(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Role'

class CourseDetail(dd.DetailLayout):
    main = "general ratings.SummariesByCourse"
    general = dd.Panel("""
    group subject
    ratings.ExamsByCourse #ratings.RatingsByCourse
    """, label=_("General"))

class Courses(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Course'
    detail_layout = "prima.CourseDetail"
    insert_layout = """
    group
    subject
    """

class CoursesByGroup(Courses):
    required_roles = dd.login_required(PrimaTeacher)
    master_key = "group"
    # label = _("Subjects given")
    order_by = ['subject']
    default_display_modes = { None: constants.DISPLAY_MODE_SUMMARY}

class CoursesBySubject(Courses):
    required_roles = dd.login_required(PrimaTeacher)
    master_key = "subject"
    # label = _("Subjects given")
    order_by = ['subject']
    default_display_modes = { None: constants.DISPLAY_MODE_SUMMARY}



class Casts(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Cast'
    detail_layout = """
    group #subject role user
    #ratings.ExamsByCast
    """

class CastsByGroup(Casts):
    required_roles = dd.login_required(PrimaTeacher)
    master_key = "group"
    # label = _("Subjects given")
    # order_by = ['subject', 'role', 'user']
    order_by = ['role', 'user']
    default_display_modes = { None: constants.DISPLAY_MODE_SUMMARY}
    insert_layout = """
    role
    # subject
    user
    """

class CastsByUser(Casts):
    required_roles = dd.login_required(PrimaTeacher)
    master_key = "user"
    # label = _("Subjects given")
    default_display_modes = { None: constants.DISPLAY_MODE_SUMMARY}
    insert_layout = """
    group
    role
    #subject
    """

# class CastsBySubject(Casts):
#     required_roles = dd.login_required(PrimaTeacher)
#     master_key = "subject"
#     column_names = "user group role *"
#     # default_display_modes = { None: constants.DISPLAY_MODE_LIST}
#     default_display_modes = { None: constants.DISPLAY_MODE_GRID}
#     insert_layout = """
#     user
#     group
#     role
#     """

class MyCasts(Casts, My):
    required_roles = dd.login_required(PrimaTeacher)
    # default_display_modes = {None: constants.DISPLAY_MODE_LIST}
    default_display_modes = { None: constants.DISPLAY_MODE_SUMMARY}
    order_by = ['group__designation', 'user', 'role']
    # order_by = ['group__designation', 'subject__seqno', 'user', 'role']
    # group_by =

    @classmethod
    def param_defaults(self, ar, **kw):
        kw = super().param_defaults(ar, **kw)
        if ar.get_user().user_type.has_required_roles([dd.SiteAdmin]):
            kw["user"] = None
        return kw

    @classmethod
    def table_as_summary(self, ar):
        groups = []
        items = []
        current_group = None
        current_casts = []

        # if ar.get_user().user_type.has_required_roles([dd.SiteAdmin]):
        #     def castf(cast):
        #         text = str(cast.role)
        #         text += " " + _("by {teacher}").format(teacher=cast.user)
        #         return text
        # else:
        #     def castf(cast):
        #         text = str(cast.role)
        #         if cast.role is not None:
        #             text += " " + _("as {role}").format(role=cast.role)
        #         # text += " " + _("in {group}").format(group=cast.group)
        #         return text

        def collect(cast):
            # current_casts.append(ar.obj2html(cast, castf(cast)))
            current_casts.append(ar.obj2html(cast))
        def end_group():
            items.append(E.li(ar.obj2html(current_group), ": ", *join_elems(current_casts, ", ")))
        for cast in ar:
            if cast.group == current_group:
                collect(cast)
                continue
            if current_group is not None:
                end_group()
                current_casts = []
            current_group = cast.group
            collect(cast)
        if current_group is not None:
            end_group()
        if len(items) == 0:
            return SAFE_EMPTY
        return tostring(E.ul(*items))

class EnrolmentDetail(dd.DetailLayout):
    main = "general ratings"
    general = dd.Panel("""
    pupil group
    projects.ProjectsByEnrolment cert.CertificatesByEnrolment
    """, _("General"))
    ratings = dd.Panel("""
    ratings.ChallengeRatingsByEnrolment ratings.FinalRatingsByEnrolment
    """, _("Ratings"))

class Enrolments(dd.Table):
    required_roles = dd.login_required(PrimaStaff)
    model = 'prima.Enrolment'
    default_display_modes = { None: constants.DISPLAY_MODE_SUMMARY}
    detail_layout = "prima.EnrolmentDetail"

class EnrolmentsByGroup(Enrolments):
    required_roles = dd.login_required(PrimaTeacher)
    master_key = "group"
    column_names = "pupil certificates *"

class EnrolmentsByPupil(Enrolments):
    required_roles = dd.login_required(PrimaPupil)
    master_key = "pupil"
    column_names = "group certificates *"



# -*- coding: UTF-8 -*-
# Copyright 2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""
The certificates plugin for Lino Prima.
"""

from lino.ad import Plugin, _


class Plugin(Plugin):

    verbose_name = _("Certificates")
    menu_group = "cert"
    needs_plugins = ['lino.modlib.printing']

    def setup_config_menu(self, site, user_type, m, ar=None):
        mg = self.get_menu_group()
        m = m.add_menu(mg.app_label, mg.verbose_name)
        m.add_action('cert.CertTemplates')

    def setup_explorer_menu(self, site, user_type, m, ar=None):
        mg = self.get_menu_group()
        m = m.add_menu(mg.app_label, mg.verbose_name)
        m.add_action('cert.CertSections')
        m.add_action('cert.CertElements')
        m.add_action('cert.Certificates')
        m.add_action('cert.SectionResponses')
        m.add_action('cert.CertRatings')



# -*- coding: UTF-8 -*-
# Copyright 2024-2025 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

from django.utils.text import format_lazy
from lino.api import dd, _
from lino.core.roles import Explorer


class ProjectStates(dd.ChoiceList):
    verbose_name = _("Project state")
    verbose_name_plural = _("Project states")

add = ProjectStates.add_item
add("1", _("Not started"), "default", button_text="☐")   # u2610 Noch nichts eingetragen
add("2", _("Started"), button_text="✎")  # u270e Teilweise eingetragen
add("3", _("Done"), button_text="☑")     # u2611 Kommentar + alle Bausteinarbeiten eingetragen.


class GeneralRating(dd.Choice):

    def __init__(self, value, text, max_score):
        self.field_name = 'gr_' + value
        self.max_score = max_score
        super().__init__(value, text)

    def get_field(self):
        verbose_name = format_lazy(_("{} ({} points)"), self.text, self.max_score)
        return dd.DecimalField(verbose_name, max_digits=4, decimal_places=1, blank=True, null=True)


class GeneralRatings(dd.ChoiceList):
    required_roles = dd.login_required(Explorer)
    item_class = GeneralRating
    verbose_name = _("General rating")
    verbose_name_plural = _("General ratings")


add = GeneralRatings.add_item
add("1", _("Cleanliness"), 3)  # Sauberkeit
add("2", _("Correction"), 3)  # Korrektur
add("3", _("Time managment"), 3)  # Zeitmanagement
add("4", _("Work behaviour"), 3)  # Arbeitsverhalten


@dd.receiver(dd.pre_analyze)
def inject_general_rating_fields(sender, **kw):
    for pf in GeneralRatings.get_list_items():
        dd.inject_field('projects.Project', pf.field_name, pf.get_field())



# -*- coding: UTF-8 -*-
# Copyright 2024-2025 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

from decimal import Decimal
from django.db import models
from django.conf import settings
from django.utils.html import format_html, mark_safe
from django.utils.text import format_lazy
from django.core.exceptions import ValidationError
from lino.utils.html import E, tostring, join_elems
from lino.utils.mldbc.mixins import BabelDesignated
from lino.modlib.users.mixins import UserAuthored, My
from lino.modlib.printing.mixins import CachedPrintable, DirectPrintAction
from lino.mixins import Referrable, Sequenced, Created
from lino_prima.lib.ratings.choicelists import RatingTypes
from lino.api import dd, rt, pgettext, _


class Grade(Referrable, BabelDesignated):  # hard-coded list in zeugnisse
    class Meta:
        verbose_name = _("Grade")  # Jahrgangsstufe, Schulstufe
        verbose_name_plural = _("Grades")
        abstract = dd.is_abstract_model(__name__, 'Grade')

    cert_template = dd.ForeignKey(
        'cert.CertTemplate', blank=True, null=True)

class Subject(BabelDesignated, Sequenced):
    class Meta:
        verbose_name = pgettext("in school", "Subject")
        verbose_name_plural = _("Subjects")
        abstract = dd.is_abstract_model(__name__, 'Subject')
    advanced = dd.BooleanField(_("Advanced"), default=False)
    icon_text = dd.CharField(_("Icon"), max_length=5, blank=True, null=True)
    rating_type = RatingTypes.field(blank=True, null=True)
    image_file = dd.ForeignKey('uploads.Upload',
        verbose_name=_("Image"), blank=True, null=True)


class Skill(BabelDesignated, Sequenced):
    class Meta:
        verbose_name = _("Skill")
        verbose_name_plural = _("Skills")
        abstract = dd.is_abstract_model(__name__, 'Skill')
    subject = dd.ForeignKey('prima.Subject')
    with_exams = dd.BooleanField(_("With final exams"))  # Mit Abschlusstests

    # def __str__(self):
    #     return f"{self.subject}:{super().__str__()}"

    def get_siblings(self):
        return self.__class__.objects.filter(subject=self.subject)



class Group(BabelDesignated):  # zeugnisse.Klasse
    class Meta:
        verbose_name = _("Group")
        verbose_name_plural = _("Groups")
        abstract = dd.is_abstract_model(__name__, 'Group')
        ordering = ['designation']
    grade = dd.ForeignKey('prima.Grade')
    year = dd.ForeignKey('periods.StoredYear', blank=True)

    @classmethod
    def get_simple_parameters(cls):
        yield super().get_simple_parameters()
        yield 'year'

    @classmethod
    def param_defaults(cls, ar, **kw):
        kw = super().param_defaults(ar, **kw)
        kw.update(year=rt.models.periods.StoredYear.get_or_create_from_date(
            dd.today()))
        return kw

    def full_clean(self, *args, **kwargs):
        if self.year_id is None:
            self.year = rt.models.periods.StoredYear.get_or_create_from_date(dd.today())
        super().full_clean(*args, **kwargs)

    def as_paragraph(self, ar, **kwargs):
        s = ar.obj2htmls(self)
        if not ar.is_obvious_field("year"):
            s += format_html(_(" ({year})"), year=str(self.year))
        return mark_safe(s)

    def __str__(self):
        s = super().__str__()
        if not self.year.covers_date(dd.today()):
            s += format_html(_(" ({year})"), year=self.year)
        return mark_safe(s)


class Course(dd.Model):
    class Meta:
        verbose_name = _("Course")
        verbose_name_plural = _("Courses")
        abstract = dd.is_abstract_model(__name__, 'Course')

    group = dd.ForeignKey('prima.Group', related_name='courses')
    subject = dd.ForeignKey('prima.Subject', related_name='courses')

    def __str__(self):
        return _("{subject} in {group}").format(
            subject=self.subject, group=self.group)

    def get_str_words(self, ar):
        if not ar.is_obvious_field("subject"):
            yield str(self.subject)
        if not ar.is_obvious_field("group"):
            yield _("in {group}").format(group=self.group)


class Role(BabelDesignated):
    class Meta:
        verbose_name = _("Role")
        verbose_name_plural = _("Roles")
        abstract = dd.is_abstract_model(__name__, 'Role')


class Cast(UserAuthored):  # zeugnisse.LehrerRolle

    class Meta:
        verbose_name = _("Cast")
        verbose_name_plural = _("Casts")
        abstract = dd.is_abstract_model(__name__, 'Cast')
        # ordering = ['user', 'group__designation', 'subject', 'role']
        ordering = ['user', 'group__designation', 'role']
        unique_together = ['user', 'group', 'role']

    group = dd.ForeignKey('prima.Group')
    # subject = dd.ForeignKey('prima.Subject')
    role = dd.ForeignKey('prima.Role', blank=True, null=True)

    def __str__(self):
        # text = str(self.subject)
        text = _("{teacher}").format(teacher=self.user)
        if self.role is not None:
            text += " " + _("as {role}").format(role=self.role)
        text += " " + _("in {group}").format(group=self.group)
        return text

    def get_str_words(self, ar):
        if self.user != ar.get_user() and not ar.is_obvious_field("user"):
            yield _("{teacher} as").format(teacher=self.user)
        yield str(self.role)
        if not ar.is_obvious_field("group"):
            yield _("in {group}").format(group=self.group)

    @classmethod
    def get_simple_parameters(cls):
        yield super().get_simple_parameters()
        yield 'group__year'

    @classmethod
    def param_defaults(cls, ar, **kw):
        kw = super().param_defaults(ar, **kw)
        kw.update(group__year=rt.models.periods.StoredYear.get_or_create_from_date(
            dd.today()))
        return kw

dd.update_field(Cast, 'user', verbose_name=_("Teacher"))


class Enrolment(dd.Model):
    class Meta:
        verbose_name = _("Enrolment")
        verbose_name_plural = _("Enrolments")
        abstract = dd.is_abstract_model(__name__, 'Enrolment')
    group = dd.ForeignKey('prima.Group')  # , related_name='enrolments')
    pupil = dd.ForeignKey('users.User', verbose_name=_("Pupil"))

    def __str__(self):
        return f"{self.pupil} ({self.group})"

    def get_str_words(self, ar):
        if not ar.is_obvious_field("pupil"):
            yield str(self.pupil)
        if not ar.is_obvious_field("group"):
            yield _("in {group}").format(group=self.group)

    # def as_summary_item(self, ar, text=None):
    #     if text is None:
    #         if ar.is_obvious_field("pupil"):
    #             text = f"{self.group}"
    #         elif ar.is_obvious_field("group"):
    #             text = f"{self.pupil}"
    #         else:
    #             text = str(self)
    #     return super().as_summary_item(ar, text)

    @dd.displayfield(_("Certificates"))
    def certificates(self, ar):
        if ar is None:
            return ''
        Certificate = rt.models.cert.Certificate
        elems = []
        insert_button_attrs = dict(style="text-align: center;")
        sar = rt.models.cert.CertificatesByEnrolment.request(parent=ar, master_instance=self)
        qs = rt.models.periods.StoredPeriod.objects.filter(year=self.group.year)
        for p in qs:
            try:
                crt = Certificate.objects.get(enrolment=self, period=p)
            except Certificate.DoesNotExist:
                btn = sar.gen_insert_button(None, insert_button_attrs,
                        enrolment=str(self), enrolmentHidden=self.pk,
                        period=str(p), periodHidden=p.pk)
                if btn is not None:
                    elems.append(btn)
                continue
            elems.append(sar.obj2html(crt, p.nickname))
        return E.p(*join_elems(elems, sep=", "))




from lino.modlib.checkdata.choicelists import Checker

class GroupChecker(Checker):
    verbose_name = _("Check for missing courses")
    model = Group
    msg_missing = _("No course for {subject} in {group}.")

    def get_checkdata_problems(self, obj, fix=False):
        CertSection = rt.models.cert.CertSection
        tpl = obj.grade.cert_template
        for cs in CertSection.objects.filter(cert_template=tpl):
            if cs.subject_id and cs.subject.advanced:
                qs = Course.objects.filter(group=obj, subject=cs.subject)
                if not qs.exists():
                    yield (True, format_lazy(
                        self.msg_missing, subject=cs.subject, group=obj))
                    if fix:
                        course = Course(subject=cs.subject, group=obj)
                        course.full_clean()
                        course.save()

GroupChecker.activate()



@dd.receiver(dd.post_analyze)
def my_details(sender, **kw):
    sender.models.system.SiteConfigs.set_detail_layout("""
    default_build_method
    simulate_today
    """)


from .ui import *



# -*- coding: UTF-8 -*-
# Copyright 2016-2025 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)

from lino.projects.std.settings import *
from lino.api import _
import lino_prima

class Site(Site):

    verbose_name = "Lino Prima"
    url = lino_prima.docs_url
    version = lino_prima.__version__

    demo_fixtures = ['std', 'demo', 'demo2', 'checkdata', 'checksummaries']
    user_types_module = 'lino_prima.lib.prima.user_types'
    migration_class = 'lino_prima.lib.prima.migrate.Migrator'
    textfield_bleached = False

    preview_limit = 40
    default_build_method = 'weasy2pdf'
    catch_layout_exceptions = False
    with_uploads = True

    def get_installed_plugins(self):
        """Implements :meth:`lino.core.site.Site.get_installed_plugins`.

        """
        yield 'bulma'
        yield 'lino.modlib.system'
        yield 'lino_prima.lib.users'
        yield 'lino_prima.lib.prima'
        yield 'lino_prima.lib.ratings'
        yield 'lino_prima.lib.projects'
        yield 'lino_prima.lib.cert'
        if self.with_uploads:
            yield 'lino.modlib.uploads'
        yield 'lino.modlib.checkdata'
        yield 'lino.modlib.summaries'
        yield 'lino.modlib.weasyprint'
        yield 'lino.modlib.export_excel'
        yield 'lino.modlib.help'
        yield super().get_installed_plugins()

    def setup_quicklinks(self, ut, tb):
        super().setup_quicklinks(ut, tb)

    def get_plugin_configs(self):
        yield super().get_plugin_configs()
        yield ('users', 'with_nickname', True)
        if self.with_uploads:
            yield ('uploads', 'with_thumbnails', True)
        yield ('periods', 'start_year', 2024)
        yield ('periods', 'year_name', _("Academic year"))
        yield ('periods', 'year_name_plural', _("Academic years"))
        yield ('periods', 'period_name', _("Period"))
        yield ('periods', 'period_name_plural', _("Periods"))
        yield ('periods', 'start_month', 9)
        yield ('periods', 'period_type', 'semester')



# -*- coding: UTF-8 -*-
# Copyright 2016-2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
"""Defines the standard user roles for Lino Prima."""

from lino.core.roles import UserRole, SiteAdmin, SiteUser, SiteStaff
from lino.modlib.users.choicelists import UserTypes
from lino.modlib.office.roles import OfficeStaff
from django.utils.translation import gettext_lazy as _

from .roles import PrimaPupil, PrimaTeacher, PrimaStaff

class Pupil(SiteUser, PrimaPupil): pass
class Teacher(SiteUser, PrimaTeacher): pass
class SiteStaff(SiteStaff, PrimaStaff, OfficeStaff): pass
class SiteAdmin(SiteAdmin, SiteStaff): pass

UserTypes.clear()
add = UserTypes.add_item
add('000',
    _("Anonymous"),
    UserRole,
    'anonymous',
    readonly=True,
    authenticated=False)
add('100', _("User"), SiteUser, 'user')
add('200', _("Teacher"), Teacher, 'teacher')
add('300', _("Pupil"), Pupil, 'pupil')
add('500', _("Staff"), SiteStaff, 'staff')
add('900', _("Administrator"), SiteAdmin, 'admin')


from django.conf import settings
settings.SITE.models.periods.PeriodTypes.semester.ref_template = "{period}"



# from django.conf import settings
# from lino.api import rt

def objects():
    # settings.SITE.site_config.update(
    #     default_build_method=rt.models.printing.BuildMethods.weasy2pdf)
    return []
